@@ -1,56 +1,74 @@
-#if IBUS_VERSION >= 10200
-/* IBus 1.2 and up */
-gboolean ibus_chewing_engine_process_key_event_1_2(IBusEngine *engine,
-	guint keysym_ignore,  guint  keycode,   guint  modifiers){
-    if (modifiers & IBUS_RELEASE_MASK){
-	/* Skip release event */
-	return TRUE;
-    }
-    IBusChewingEngine *self=IBUS_CHEWING_ENGINE(engine);
+/**
+ * ibus_chewing_engine_keycode_to_keysym:
+ *
+ * Convert keycode to keysym.
+ */
+guint ibus_chewing_engine_keycode_to_keysym(IBusChewingEngine *self, guint keysym,  guint keycode, guint modifiers){
+    /* Get system layout */
     GValue gValue={0};
     gboolean useSysKeyLayout=TRUE;
     if (ibus_chewing_config_get_value(self->config, "general", "use_system_keyboard_layout", &gValue)){
 	useSysKeyLayout=g_value_get_boolean(&gValue);
     }
-    guint keysym;
+
+    guint kSym;
     if (useSysKeyLayout && (!chewing_get_ChiEngMode(self->context))){
 	// English mode.
-	keysym=keysym_ignore;
+	kSym=keysym;
     }else{
-	keysym=ibus_keymap_lookup_keysym (self->keymap_us,keycode,modifiers);
+	/* ibus_keymap_lookup_keysym only handles keycode under 256 */
+	if ((kSym=ibus_keymap_lookup_keysym(self->keymap_us,keycode,modifiers))==IBUS_VoidSymbol){
+	    kSym=keysym;
+	}
     }
-
-    return ibus_chewing_engine_process_key_event(engine, keysym, modifiers);
+    return kSym;
 }
-#endif
 
 gboolean ibus_chewing_engine_process_key_event(IBusEngine *engine,
-	guint keysym, guint  modifiers){
+	guint keysym,  guint keycode, guint modifiers){
     gboolean result=TRUE;
+
+    IBusChewingEngine *self=IBUS_CHEWING_ENGINE(engine);
+    if (ibus_chewing_engine_is_password(self)) return FALSE;
+    guint kSym=ibus_chewing_engine_keycode_to_keysym(self,keysym, keycode, modifiers);
+
     if (modifiers & IBUS_RELEASE_MASK){
+	if (!keysym_KP_to_normal(kSym) && (kSym==IBUS_Shift_L || kSym==IBUS_Shift_R) && self->_priv->key_last==kSym){
+	    /* When Chi->Eng with incomplete character */
+	    if (chewing_get_ChiEngMode(self->context) && !chewing_zuin_Check(self->context)){
+	        /* chewing_zuin_Check==0 means incomplete character */
+	        /* Send a space to finish the character */
+	        chewing_handle_Space(self->context);
+	     }
+             chewing_set_ChiEngMode(self->context, ! chewing_get_ChiEngMode(self->context));
+	     self_refresh_property(self,"chewing_chieng_prop");
+	     return self_update(self);
+	}
 	/* Skip release event */
 	return TRUE;
     }
-    IBusChewingEngine *self=IBUS_CHEWING_ENGINE(engine);
-    G_DEBUG_MSG(2,"***[I2] process_key_event(-, %u(%s), %u) ... proceed.",keysym, keyName_get(keysym), modifiers);
+
+    IBUS_CHEWING_LOG(2,"***[I2] process_key_event(-, %x(%s), %x, %x) orig keysym=%x... proceed.",kSym, keyName_get(kSym), keycode, modifiers,keysym);
     guint state= modifiers & (IBUS_SHIFT_MASK | IBUS_CONTROL_MASK | IBUS_MOD1_MASK);
-    self->_priv->key_last=keysym;
+    self->_priv->key_last=kSym;
     if (state==0){
-	guint keysym_tmp=keysym_KP_to_normal(keysym);
-	if (keysym_tmp){
+	guint kSym_tmp=keysym_KP_to_normal(kSym);
+	if (kSym_tmp){
+	    IBUS_CHEWING_LOG(3,"***[I3] process_key_event(): %x is from keypad.", kSym_tmp);
 	    /* Is keypad key */
 	    if ((self->chewingFlags & CHEWING_FLAG_NUMPAD_ALWAYS_NUMBER) && chewing_get_ChiEngMode(self->context)){
 		chewing_set_ChiEngMode(self->context, 0);
-		self_handle_Default(self,keysym_tmp,FALSE);
+		self_handle_Default(self,kSym_tmp,FALSE);
 		chewing_set_ChiEngMode(self->context,CHINESE_MODE);
 	    }else{
 		/* Convert kp numbers to normal */
-		self_handle_Default(self,keysym_tmp,FALSE);
+		self_handle_Default(self,kSym_tmp,FALSE);
 	    }
 	}else{
-	    switch (keysym){
+	    switch (kSym){
 		case IBUS_Return:
 		case IBUS_KP_Enter:
+		    ibus_chewing_engine_set_status_flag(self,ENGINE_STATUS_NEED_COMMIT);
 		    chewing_handle_Enter(self->context);
 		    break;
 		case IBUS_Escape:
@@ -71,12 +89,20 @@ gboolean ibus_chewing_engine_process_key_event(IBusEngine *engine,
 		    break;
 		case IBUS_space:
 		case IBUS_KP_Space:
-		    /**
-		     * Fix for space in Temporary English mode.
-		     */
-		    chewing_handle_Space(self->context);
+		    if (self->chewingFlags & CHEWING_FLAG_PLAIN_ZHUYIN) {
+			if (chewing_cand_TotalChoice(self->context) == 0) {
+			    chewing_handle_Space(self->context);
+			}
+		    } else {
+			/**
+			 * Fix for space in Temporary English mode.
+			 */
+			chewing_handle_Space(self->context);
+			ibus_chewing_engine_set_status_flag(self,ENGINE_STATUS_NEED_COMMIT);
+		    }
 		    if (self->inputMode==CHEWING_INPUT_MODE_SELECTION_DONE ||
-			    self->inputMode==CHEWING_INPUT_MODE_BYPASS )
+			    self->inputMode==CHEWING_INPUT_MODE_BYPASS ||
+			    self->inputMode==CHEWING_INPUT_MODE_EDITING )
 			ibus_chewing_engine_set_status_flag(self,ENGINE_STATUS_NEED_COMMIT);
 		    break;
 		case IBUS_Page_Up:
@@ -155,12 +181,18 @@ gboolean ibus_chewing_engine_process_key_event(IBusEngine *engine,
 		    /* Some QT application will sneak these through */
 		    return FALSE;
 		default:
-		    self_handle_Default(self,keysym,FALSE);
+		    self_handle_Default(self,kSym,FALSE);
 		    break;
 	    }
 	}
     }else if (state==IBUS_SHIFT_MASK){
-	switch(keysym){
+	switch(kSym){
+	    case IBUS_Return:
+	    case IBUS_KP_Enter:
+		/* Same with Shift Enter and Shift KP_Enter */
+		ibus_chewing_engine_set_status_flag(self,ENGINE_STATUS_NEED_COMMIT);
+		chewing_handle_Enter(self->context);
+		break;
 	    case IBUS_Left:
 		if (self->inputMode==CHEWING_INPUT_MODE_BYPASS)
 		    return FALSE;
@@ -191,17 +223,17 @@ gboolean ibus_chewing_engine_process_key_event(IBusEngine *engine,
 		self_refresh_property(self,"chewing_alnumSize_prop");
 		break;
 	    default:
-		if (keysym>127 || keysym<0){
+		if (kSym>127 || kSym<0){
 		    /* Special keys, must let it through */
 		    return FALSE;
 		}
-		self_handle_Default(self,keysym,TRUE);
+		self_handle_Default(self,kSym,TRUE);
 		break;
 	}
     }else if (state==IBUS_CONTROL_MASK){
-	if (keysym>=IBUS_0 && keysym<=IBUS_9){
-	    chewing_handle_CtrlNum(self->context,keysym);
-//	}else if (keysym==IBUS_v || keysym==IBUS_V){
+	if (kSym>=IBUS_0 && kSym<=IBUS_9){
+	    chewing_handle_CtrlNum(self->context,kSym);
+//	}else if (kSym==IBUS_v || kSym==IBUS_V){
 //	    chewing_handle_Right(self->context);
 	}else{
 	    result=FALSE;
@@ -217,7 +249,7 @@ gboolean ibus_chewing_engine_process_key_event(IBusEngine *engine,
 }
 
 void ibus_chewing_engine_handle_Default(IBusChewingEngine *self, guint keyval, gboolean shiftPressed){
-    G_DEBUG_MSG(2,"[I2] handle_Default(-,%u) plainZhuyin=%s inputMode=%d",
+    IBUS_CHEWING_LOG(2,"[I2] handle_Default(-,%u) plainZhuyin=%s inputMode=%d",
 	    keyval,(self->chewingFlags & CHEWING_FLAG_PLAIN_ZHUYIN)? "TRUE": "FALSE",self->inputMode);
     ibus_chewing_engine_set_status_flag(self, ENGINE_STATUS_NEED_COMMIT);
 #ifdef EASY_SYMBOL_INPUT_WORK_AROUND
@@ -249,10 +281,11 @@ void ibus_chewing_engine_handle_Default(IBusChewingEngine *self, guint keyval, g
  * Mouse events
  */
 void ibus_chewing_engine_candidate_clicked(IBusEngine *engine, guint index, guint button, guint state){
-    G_DEBUG_MSG(2,"***[I2] candidate_clicked(-, %u, %u, %u) ... proceed.", index, button, state);
+    IBUS_CHEWING_LOG(2,"***[I2] candidate_clicked(-, %u, %u, %u) ... proceed.", index, button, state);
     IBusChewingEngine *self=IBUS_CHEWING_ENGINE(engine);
+    if (ibus_chewing_engine_is_password(self)) return;
     if (index >= chewing_get_candPerPage(self->context) || index <0) {
-	G_DEBUG_MSG(3,"[I3]  candidate_clicked() index out of ranged");
+	IBUS_CHEWING_LOG(3,"[I3]  candidate_clicked() index out of ranged");
 	return;
     }
     if (self->inputMode==CHEWING_INPUT_MODE_SELECTING){
@@ -260,12 +293,12 @@ void ibus_chewing_engine_candidate_clicked(IBusEngine *engine, guint index, guin
 	ibus_chewing_engine_handle_Default(self, self->_priv->key_last, FALSE);
 	self_update(self);
     } else {
-	G_DEBUG_MSG(3,"[I3] candidate_clicked() ... Wrong mode: %u", self->inputMode);
+	IBUS_CHEWING_LOG(3,"[I3] candidate_clicked() ... Wrong mode: %u", self->inputMode);
     }
 }
 
 void ibus_chewing_engine_property_activate(IBusEngine *engine, const gchar  *prop_name, guint  prop_state){
-    G_DEBUG_MSG(3,"[I3] property_activate(-, %s, %u)", prop_name, prop_state);
+    IBUS_CHEWING_LOG(3,"[I3] property_activate(-, %s, %u)", prop_name, prop_state);
     Self *self=SELF(engine);
     gboolean needRefresh=TRUE;
     if (strcmp(prop_name,"chewing_chieng_prop")==0){
@@ -275,17 +308,40 @@ void ibus_chewing_engine_property_activate(IBusEngine *engine, const gchar  *pro
 	/* Toggle Full <-> Half */
 	chewing_set_ShapeMode(self->context, !chewing_get_ShapeMode(self->context));
     }else if (strcmp(prop_name,"chewing_settings_prop")==0){
-	if (self->settings_prop->state==PROP_STATE_UNCHECKED){
+#if IBUS_CHECK_VERSION(1, 4, 0)
+	if (ibus_property_get_state(self->settings_prop)==PROP_STATE_UNCHECKED)
+#else
+	if (self->settings_prop->state==PROP_STATE_UNCHECKED)
+#endif
+	{
 	    if (gtk_dialog_run(GTK_DIALOG(self->setting_dialog))==GTK_RESPONSE_OK){
 		self_save_config_all(self);
 	    }
 	    gtk_widget_hide(self->setting_dialog);
+#if IBUS_CHECK_VERSION(1, 4, 0)
+		ibus_property_set_state(self->settings_prop,PROP_STATE_UNCHECKED);
+#else
 	    self->settings_prop->state=PROP_STATE_UNCHECKED;
+#endif
 	}
     }else{
-	G_DEBUG_MSG(3,"[I3]  property_activate(-, %s, %u) not recognized",prop_name, prop_state);
+	IBUS_CHEWING_LOG(3,"[I3]  property_activate(-, %s, %u) not recognized",prop_name, prop_state);
 	needRefresh=FALSE;
     }
     if (needRefresh)
 	self_refresh_property(self,prop_name);
 }
+
+#if IBUS_CHECK_VERSION(1, 5, 4)
+void ibus_chewing_engine_set_content_type(IBusEngine *engine, guint purpose, guint hints){
+    IBUS_CHEWING_LOG(5,"[I5] set_content_type(%d, %d)", purpose, hints);
+
+    Self *self=SELF(engine);
+    if (purpose == IBUS_INPUT_PURPOSE_PASSWORD ||
+	purpose == IBUS_INPUT_PURPOSE_PIN) {
+	ibus_chewing_engine_set_status_flag(self, ENGINE_STATUS_IS_PASSWORD);
+    } else {
+	ibus_chewing_engine_clear_status_flag(self, ENGINE_STATUS_IS_PASSWORD);
+    }
+}
+#endif
